@@ -9,6 +9,7 @@ Setting audio device:
 
 - `set_device()`
 - `print_device_info()`
+- `default_config`
 
 Playing audio:
 
@@ -32,19 +33,30 @@ References
 - https://pypi.org/project/sounddevice/
 
 """
-from .audio_io import (play, play_and_record, set_device, record,
-                       print_device_info, play_through_stream, CallbackStop,
-                       standard_callback, sleep, output_stream)
+from .audio_io import (
+    play,
+    play_and_record,
+    set_device,
+    record,
+    print_device_info,
+    play_through_stream,
+    CallbackStop,
+    standard_callback,
+    sleep,
+    output_stream,
+    default_config,
+)
 
 __all__ = [
-    'play',
-    'play_and_record',
-    'set_device',
-    'record',
-    'print_device_info',
-    'play_through_stream',
-    'CallbackStop',
-    'standard_callback',
-    'sleep',
-    'output_stream',
+    "play",
+    "play_and_record",
+    "set_device",
+    "record",
+    "print_device_info",
+    "play_through_stream",
+    "CallbackStop",
+    "standard_callback",
+    "sleep",
+    "output_stream",
+    "default_config",
 ]
