@@ -49,18 +49,18 @@ from .beamforming import (
 )
 
 __all__ = [
-    'Grid',
-    'Regular2DGrid',
-    'Regular3DGrid',
-    'LineGrid',
-    'MicArray',
-    'SteeringVector',
-    'BeamformerDASFrequency',
-    'BeamformerCleanSC',
-    'BeamformerOrthogonal',
-    'BeamformerFunctional',
-    'BeamformerMVDR',
-    'BeamformerDASTime',
-    'MonopoleSource',
-    'mix_sources_on_array',
+    "Grid",
+    "Regular2DGrid",
+    "Regular3DGrid",
+    "LineGrid",
+    "MicArray",
+    "SteeringVector",
+    "BeamformerDASFrequency",
+    "BeamformerCleanSC",
+    "BeamformerOrthogonal",
+    "BeamformerFunctional",
+    "BeamformerMVDR",
+    "BeamformerDASTime",
+    "MonopoleSource",
+    "mix_sources_on_array",
 ]
