@@ -7,22 +7,30 @@ from enum import Enum
 import scipy.signal as sig
 from .signal_class import Signal
 from .multibandsignal import MultiBandSignal
-from dsptoolbox._general_helpers import _polyphase_decomposition
+from .._general_helpers import _polyphase_decomposition
 
 
-def _get_biquad_type(number: int = None, name: str = None):
-    """Helper method that handles string inputs for the biquad filters.
-
-    """
+def _get_biquad_type(number: int | None = None, name: str | None = None):
+    """Helper method that handles string inputs for the biquad filters."""
     if name is not None:
         name = name.lower()
-        valid_names = ('peaking', 'lowpass', 'highpass', 'bandpass_skirt',
-                       'bandpass_peak', 'notch', 'allpass', 'lowshelf',
-                       'highshelf')
-        assert name in valid_names, f'{name} is not a valid name. Please ' +\
-            '''select from the ('peaking', 'lowpass', 'highpass',
+        valid_names = (
+            "peaking",
+            "lowpass",
+            "highpass",
+            "bandpass_skirt",
+            "bandpass_peak",
+            "notch",
+            "allpass",
+            "lowshelf",
+            "highshelf",
+        )
+        assert name in valid_names, (
+            f"{name} is not a valid name. Please "
+            + """select from the ('peaking', 'lowpass', 'highpass',
             'bandpass_skirt', 'bandpass_peak', 'notch', 'allpass', 'lowshelf',
-            'highshelf')'''
+            'highshelf')"""
+        )
 
     class biquad(Enum):
         peaking = 0
@@ -36,19 +44,24 @@ def _get_biquad_type(number: int = None, name: str = None):
         highshelf = 8
 
     if number is None:
-        assert name is not None, \
-            'Either number or name must be given, not both'
-        r = eval(f'biquad.{name}')
+        assert (
+            name is not None
+        ), "Either number or name must be given, not both"
+        r = eval(f"biquad.{name}")
         r = r.value
     else:
-        assert name is None, 'Either number or name must be given, not both'
+        assert name is None, "Either number or name must be given, not both"
         r = biquad(number).name
     return r
 
 
-def _biquad_coefficients(eq_type: int | str = 0, fs_hz: int = 48000,
-                         frequency_hz: float = 1000, gain_db: float = 1,
-                         q: float = 1):
+def _biquad_coefficients(
+    eq_type: int | str = 0,
+    fs_hz: int = 48000,
+    frequency_hz: float = 1000,
+    gain_db: float = 1,
+    q: float = 1,
+):
     """Creates the filter coefficients for biquad filters.
     eq_type: 0 PEAKING, 1 LOWPASS, 2 HIGHPASS, 3 BANDPASS_SKIRT,
         4 BANDPASS_PEAK, 5 NOTCH, 6 ALLPASS, 7 LOWSHELF, 8 HIGHSHELF.
@@ -65,10 +78,12 @@ def _biquad_coefficients(eq_type: int | str = 0, fs_hz: int = 48000,
     frequency_hz = np.asarray(frequency_hz)
     if frequency_hz.ndim > 0:
         frequency_hz = np.mean(frequency_hz)
-        warn('More than one frequency was passed for biquad filter. This is ' +
-             'not supported. A mean of passed frequencies was used for the ' +
-             'design but this might not give the intended result!')
-    A = np.sqrt(10**(gain_db / 20.0))
+        warn(
+            "More than one frequency was passed for biquad filter. This is "
+            + "not supported. A mean of passed frequencies was used for the "
+            + "design but this might not give the intended result!"
+        )
+    A = np.sqrt(10 ** (gain_db / 20.0))
     Omega = 2.0 * np.pi * (frequency_hz / fs_hz)
     sn = np.sin(Omega)
     cs = np.cos(Omega)
@@ -77,7 +92,7 @@ def _biquad_coefficients(eq_type: int | str = 0, fs_hz: int = 48000,
     b = np.ones(3, dtype=np.float32)
     if eq_type == 0:  # Peaking
         b[0] = 1 + alpha * A
-        b[1] = -2*cs
+        b[1] = -2 * cs
         b[2] = 1 - alpha * A
         a[0] = 1 + alpha / A
         a[1] = -2 * cs
@@ -125,21 +140,21 @@ def _biquad_coefficients(eq_type: int | str = 0, fs_hz: int = 48000,
         a[1] = -2 * cs
         a[2] = 1 - alpha
     elif eq_type == 7:  # Lowshelf
-        b[0] = A*((A+1) - (A-1)*cs + 2*np.sqrt(A)*alpha)
-        b[1] = 2*A*((A-1) - (A+1)*cs)
-        b[2] = A*((A+1) - (A-1)*cs - 2*np.sqrt(A)*alpha)
-        a[0] = (A+1) + (A-1)*cs + 2*np.sqrt(A)*alpha
-        a[1] = -2*((A-1) + (A+1)*cs)
-        a[2] = (A+1) + (A-1)*cs - 2*np.sqrt(A)*alpha
+        b[0] = A * ((A + 1) - (A - 1) * cs + 2 * np.sqrt(A) * alpha)
+        b[1] = 2 * A * ((A - 1) - (A + 1) * cs)
+        b[2] = A * ((A + 1) - (A - 1) * cs - 2 * np.sqrt(A) * alpha)
+        a[0] = (A + 1) + (A - 1) * cs + 2 * np.sqrt(A) * alpha
+        a[1] = -2 * ((A - 1) + (A + 1) * cs)
+        a[2] = (A + 1) + (A - 1) * cs - 2 * np.sqrt(A) * alpha
     elif eq_type == 8:  # Highshelf
-        b[0] = A*((A+1) + (A-1)*cs + 2*np.sqrt(A)*alpha)
-        b[1] = -2*A*((A-1) + (A+1)*cs)
-        b[2] = A*((A+1) + (A-1)*cs - 2*np.sqrt(A)*alpha)
-        a[0] = (A+1) - (A-1)*cs + 2*np.sqrt(A)*alpha
-        a[1] = 2*((A-1) - (A+1)*cs)
-        a[2] = (A+1) - (A-1)*cs - 2*np.sqrt(A)*alpha
+        b[0] = A * ((A + 1) + (A - 1) * cs + 2 * np.sqrt(A) * alpha)
+        b[1] = -2 * A * ((A - 1) + (A + 1) * cs)
+        b[2] = A * ((A + 1) + (A - 1) * cs - 2 * np.sqrt(A) * alpha)
+        a[0] = (A + 1) - (A - 1) * cs + 2 * np.sqrt(A) * alpha
+        a[1] = 2 * ((A - 1) - (A + 1) * cs)
+        a[2] = (A + 1) - (A - 1) * cs - 2 * np.sqrt(A) * alpha
     else:
-        raise Exception('eq_type not supported')
+        raise Exception("eq_type not supported")
     return b, a
 
 
@@ -192,22 +207,27 @@ def _group_delay_filter(ba, length_samples: int = 512, fs_hz: int = 48000):
     c = np.convolve(ba[0], np.conjugate(ba[1][::-1]))
     cr = c * np.arange(len(c))  # Ramped coefficients
     # Evaluation
-    num = np.polyval(cr, np.exp(1j*omega))
-    denum = np.polyval(c, np.exp(1j*omega))
+    num = np.polyval(cr, np.exp(1j * omega))
+    denum = np.polyval(c, np.exp(1j * omega))
 
     # Group delay
-    gd = np.real(num/denum) - len(ba[1]) + 1
+    gd = np.real(num / denum) - len(ba[1]) + 1
 
     # Look for infinite values
     gd[~np.isfinite(gd)] = 0
-    f = omega/np.pi*(fs_hz/2)
+    f = omega / np.pi * (fs_hz / 2)
     gd /= fs_hz
     return f, gd
 
 
-def _filter_on_signal(signal: Signal, sos, channels=None,
-                      zi=None, zero_phase: bool = False,
-                      warning_on_complex_output: bool = True):
+def _filter_on_signal(
+    signal: Signal,
+    sos,
+    channels=None,
+    zi=None,
+    zero_phase: bool = False,
+    warning_on_complex_output: bool = True,
+):
     """Takes in a `Signal` object and filters selected channels. Exports a new
     `Signal` object.
 
@@ -251,10 +271,9 @@ def _filter_on_signal(signal: Signal, sos, channels=None,
 
     # Filtering
     if zi is not None:
-        y, zi[:, :, channels] = \
-            sig.sosfilt(
-                sos, signal.time_data[:, channels],
-                zi=zi[:, :, channels], axis=0)
+        y, zi[:, :, channels] = sig.sosfilt(
+            sos, signal.time_data[:, channels], zi=zi[:, :, channels], axis=0
+        )
     else:
         if zero_phase:
             y = sig.sosfiltfilt(sos, signal.time_data[:, channels], axis=0)
@@ -264,9 +283,11 @@ def _filter_on_signal(signal: Signal, sos, channels=None,
     # Check for complex output
     if np.iscomplexobj(y):
         if warning_on_complex_output:
-            warn('Filter output is complex. Imaginary part is saved in ' +
-                 'Signal as time_data_imaginary')
-        new_time_data = new_time_data.astype('cfloat')
+            warn(
+                "Filter output is complex. Imaginary part is saved in "
+                + "Signal as time_data_imaginary"
+            )
+        new_time_data = new_time_data.astype("cfloat")
 
     # Create new signal
     new_time_data[:, channels] = y
@@ -281,10 +302,15 @@ def _filter_on_signal(signal: Signal, sos, channels=None,
     return new_signal, zi
 
 
-def _filter_on_signal_ba(signal: Signal, ba, channels=None,
-                         zi: list = None, zero_phase: bool = False,
-                         filter_type: str = 'iir',
-                         warning_on_complex_output: bool = True):
+def _filter_on_signal_ba(
+    signal: Signal,
+    ba,
+    channels=None,
+    zi: list | None = None,
+    zero_phase: bool = False,
+    filter_type: str = "iir",
+    warning_on_complex_output: bool = True,
+):
     """Takes in a `Signal` object and filters selected channels. Exports a new
     `Signal` object.
 
@@ -322,13 +348,14 @@ def _filter_on_signal_ba(signal: Signal, ba, channels=None,
     """
     # Take lfilter function, might be a different one depending if filter is
     # FIR or IIR
-    if filter_type == 'fir':
+    if filter_type == "fir":
         lfilter = _lfilter_fir
-    elif filter_type in ('iir', 'biquad'):
+    elif filter_type in ("iir", "biquad"):
         lfilter = sig.lfilter
     else:
         raise ValueError(
-            f'{filter_type} is not supported. Use either fir or iir')
+            f"{filter_type} is not supported. Use either fir or iir"
+        )
 
     # Time Data
     new_time_data = signal.time_data
@@ -344,22 +371,30 @@ def _filter_on_signal_ba(signal: Signal, ba, channels=None,
     # Filtering
     if zi is not None:
         y, zi[:, channels] = lfilter(
-                ba[0], a=ba[1], x=signal.time_data[:, channels],
-                zi=zi[:, channels])
+            ba[0],
+            a=ba[1],
+            x=signal.time_data[:, channels],
+            zi=zi[:, channels],
+            axis=0,
+        )
     else:
         if zero_phase:
             y = sig.filtfilt(
-                b=ba[0], a=ba[1], x=signal.time_data[:, channels], axis=0)
+                b=ba[0], a=ba[1], x=signal.time_data[:, channels], axis=0
+            )
         else:
             y = lfilter(
-                ba[0], a=ba[1], x=signal.time_data[:, channels])
+                ba[0], a=ba[1], x=signal.time_data[:, channels], axis=0
+            )
 
     # Check for complex output
     if np.iscomplexobj(y):
         if warning_on_complex_output:
-            warn('Filter output is complex. Imaginary part is saved in ' +
-                 'Signal as time_data_imaginary')
-        new_time_data = new_time_data.astype('cfloat')
+            warn(
+                "Filter output is complex. Imaginary part is saved in "
+                + "Signal as time_data_imaginary"
+            )
+        new_time_data = new_time_data.astype("cfloat")
 
     # Create new signal
     new_time_data[:, channels] = y
@@ -374,9 +409,14 @@ def _filter_on_signal_ba(signal: Signal, ba, channels=None,
     return new_signal, zi
 
 
-def _filterbank_on_signal(signal: Signal, filters, activate_zi: bool = False,
-                          mode: str = 'parallel', zero_phase: bool = False,
-                          same_sampling_rate: bool = True):
+def _filterbank_on_signal(
+    signal: Signal,
+    filters,
+    activate_zi: bool = False,
+    mode: str = "parallel",
+    zero_phase: bool = False,
+    same_sampling_rate: bool = True,
+):
     """Applies filter bank on a given signal.
 
     Parameters
@@ -405,27 +445,29 @@ def _filterbank_on_signal(signal: Signal, filters, activate_zi: bool = False,
 
     """
     n_filt = len(filters)
-    if mode == 'parallel':
+    if mode == "parallel":
         ss = []
         for n in range(n_filt):
             ss.append(
                 filters[n].filter_signal(
-                    signal, activate_zi=activate_zi, zero_phase=zero_phase))
-        out_sig = MultiBandSignal(
-            ss, same_sampling_rate=same_sampling_rate)
-    elif mode == 'sequential':
+                    signal, activate_zi=activate_zi, zero_phase=zero_phase
+                )
+            )
+        out_sig = MultiBandSignal(ss, same_sampling_rate=same_sampling_rate)
+    elif mode == "sequential":
         out_sig = signal.copy()
         for n in range(n_filt):
-            out_sig = \
-                filters[n].filter_signal(
-                    out_sig, activate_zi=activate_zi, zero_phase=zero_phase)
+            out_sig = filters[n].filter_signal(
+                out_sig, activate_zi=activate_zi, zero_phase=zero_phase
+            )
     else:
-        new_time_data = \
-            np.zeros((signal.time_data.shape[0],
-                      signal.number_of_channels, n_filt))
+        new_time_data = np.zeros(
+            (signal.time_data.shape[0], signal.number_of_channels, n_filt)
+        )
         for n in range(n_filt):
             s = filters[n].filter_signal(
-                    signal, activate_zi=activate_zi, zero_phase=zero_phase)
+                signal, activate_zi=activate_zi, zero_phase=zero_phase
+            )
             new_time_data[:, :, n] = s.time_data
         new_time_data = np.sum(new_time_data, axis=-1)
         out_sig = signal.copy()
@@ -433,63 +475,69 @@ def _filterbank_on_signal(signal: Signal, filters, activate_zi: bool = False,
     return out_sig
 
 
-def _lfilter_fir(b: np.ndarray, a: np.ndarray, x: np.ndarray,
-                 zi: np.ndarray = None):
+def _lfilter_fir(
+    b: np.ndarray,
+    a: np.ndarray,
+    x: np.ndarray,
+    zi: np.ndarray | None = None,
+    axis: int = 0,
+):
     """Variant to the `scipy.signal.lfilter` that uses `scipy.signal.convolve`
     for filtering. The advantage of this is that the convolution will be
     automatically made using fft or direct, depending on the inputs' sizes.
     This is only used for FIR filters.
 
+    The `axis` parameter is only there for compatibility with
+    `scipy.signal.lfilter`, but the first axis is always used.
+
     """
-    assert len(a) == 1, \
-        f'{a} is not valid. It has to be 1 in order to be a valid FIR filter'
+    assert (
+        len(a) == 1
+    ), f"{a} is not valid. It has to be 1 in order to be a valid FIR filter"
 
     # b dimensions handling
     if b.ndim != 1:
         b = np.squeeze(b)
-        assert b.ndim == 1, \
-            'FIR Filters for audio must be 1D-arrays'
+        assert b.ndim == 1, "FIR Filters for audio must be 1D-arrays"
 
     # Dimensions of zi and x must match
     if zi is not None:
-        assert zi.ndim == x.ndim, \
-            'Vector to filter and initial values should have the same ' +\
-            'number of dimensions!'
+        assert zi.ndim == x.ndim, (
+            "Vector to filter and initial values should have the same "
+            + "number of dimensions!"
+        )
     if x.ndim < 2:
         x = x[..., None]
         if zi is not None:
             zi = zi[..., None]
-    assert x.ndim == 2, \
-        'Filtering only works on 2D-arrays'
+    assert x.ndim == 2, "Filtering only works on 2D-arrays"
 
     # Convolving
-    y = sig.convolve(x, b[..., None], mode='full')
+    y = sig.convolve(x, b[..., None], mode="full")
 
     # Use zi's and take zf's
     if zi is not None:
-        y[:zi.shape[0], :] += zi
-        zf = y[-zi.shape[0]:, :]
+        y[: zi.shape[0], :] += zi
+        zf = y[-zi.shape[0] :, :]
 
     # Trim output
-    y = y[:x.shape[0], :]
+    y = y[: x.shape[0], :]
     if zi is None:
         return y
     return y, zf
 
 
-def _filter_and_downsample(time_data: np.ndarray, down_factor: int,
-                           ba_coefficients: list, polyphase: bool) \
-        -> np.ndarray:
+def _filter_and_downsample(
+    time_data: np.ndarray,
+    down_factor: int,
+    ba_coefficients: list,
+    polyphase: bool,
+) -> np.ndarray:
     """Filters and downsamples time data. If polyphase is `True`, it is
     assumed that the filter is FIR and only b-coefficients are used. In
     that case, an efficient downsampling is done, otherwise standard filtering
     and downsampling is applied.
 
-    NOTE: There must be a bug with this implementation because it is not
-    exactly the same output as when one uses normal filtering or
-    `scipy.signal.resample_poly`. Perceptually there is no difference between
-    the two results, though.
-
     Parameters
     ----------
     time_data : `np.ndarray`
@@ -511,8 +559,9 @@ def _filter_and_downsample(time_data: np.ndarray, down_factor: int,
     """
     if time_data.ndim == 1:
         time_data = time_data[..., None]
-    assert time_data.ndim == 2, \
-        'Shape for time data should be (time samples, channels)'
+    assert (
+        time_data.ndim == 2
+    ), "Shape for time data should be (time samples, channels)"
 
     if polyphase:
         poly, _ = _polyphase_decomposition(time_data, down_factor, flip=False)
@@ -522,7 +571,8 @@ def _filter_and_downsample(time_data: np.ndarray, down_factor: int,
         half_length = (len(b) - 1) // 2
         b_poly, _ = _polyphase_decomposition(b, down_factor, flip=True)
         new_time_data = np.zeros(
-            (poly.shape[0]+b_poly.shape[0]-1, poly.shape[2]))
+            (poly.shape[0] + b_poly.shape[0] - 1, poly.shape[2])
+        )
         # Accumulator for each channel – it would be better to find a way
         # to do it without loops, but using scipy.signal.convolve since it
         # is advantageous compared to numpy.convolve
@@ -530,22 +580,28 @@ def _filter_and_downsample(time_data: np.ndarray, down_factor: int,
             temp = np.zeros(new_time_data.shape[0])
             for n in range(poly.shape[1]):
                 temp += sig.convolve(
-                    poly[:, n, ch], b_poly[:, n, 0], mode='full')
+                    poly[:, n, ch], b_poly[:, n, 0], mode="full"
+                )
             new_time_data[:, ch] = temp
         # Take correct values from vector
-        new_time_data = \
-            new_time_data[
-                half_length//down_factor:-half_length//down_factor, :]
+        new_time_data = new_time_data[
+            half_length // down_factor : -half_length // down_factor, :
+        ]
     else:
         new_time_data = sig.lfilter(
-            ba_coefficients[0], ba_coefficients[1], x=time_data, axis=0)
+            ba_coefficients[0], ba_coefficients[1], x=time_data, axis=0
+        )
         new_time_data = new_time_data[::down_factor]
 
     return new_time_data
 
 
-def _filter_and_upsample(time_data: np.ndarray, up_factor: int,
-                         ba_coefficients: list, polyphase: bool):
+def _filter_and_upsample(
+    time_data: np.ndarray,
+    up_factor: int,
+    ba_coefficients: list,
+    polyphase: bool,
+):
     """Filters and upsamples time data. If polyphase is `True`, it is
     assumed that the filter is FIR and only b-coefficients are used. In
     that case, an efficient polyphase upsampling is done, otherwise standard
@@ -576,8 +632,9 @@ def _filter_and_upsample(time_data: np.ndarray, up_factor: int,
     """
     if time_data.ndim == 1:
         time_data = time_data[..., None]
-    assert time_data.ndim == 2, \
-        'Shape for time data should be (time samples, channels)'
+    assert (
+        time_data.ndim == 2
+    ), "Shape for time data should be (time samples, channels)"
 
     if polyphase:
         b = ba_coefficients[0]
@@ -589,26 +646,33 @@ def _filter_and_upsample(time_data: np.ndarray, up_factor: int,
 
         # Accumulator – Length is not right!
         new_time_data = np.zeros(
-            ((time_data.shape[0] + b_poly.shape[0] - 1)*up_factor,
-             time_data.shape[1]))
+            (
+                (time_data.shape[0] + b_poly.shape[0] - 1) * up_factor,
+                time_data.shape[1],
+            )
+        )
 
         # Interpolate per channel and per polyphase component – should be
         # a better way to do it without the loops...
         for ch in range(time_data.shape[1]):
             for ind in range(up_factor):
                 new_time_data[ind::up_factor, ch] = sig.convolve(
-                        time_data[:, ch], b_poly[:, ind, 0], mode='full')
+                    time_data[:, ch], b_poly[:, ind, 0], mode="full"
+                )
 
         # Take right samples from filtered signal
         if padding == up_factor:
             new_time_data = new_time_data[half_length:-half_length, :]
         else:
-            new_time_data = \
-                new_time_data[half_length+padding:-half_length+padding, :]
+            new_time_data = new_time_data[
+                half_length + padding : -half_length + padding, :
+            ]
     else:
         new_time_data = np.zeros(
-            (time_data.shape[0]*up_factor, time_data.shape[1]))
+            (time_data.shape[0] * up_factor, time_data.shape[1])
+        )
         new_time_data[::up_factor] = time_data
         new_time_data = sig.lfilter(
-            ba_coefficients[0], ba_coefficients[1], x=new_time_data, axis=0)
+            ba_coefficients[0], ba_coefficients[1], x=new_time_data, axis=0
+        )
     return new_time_data
