@@ -9,21 +9,25 @@ from .signal_class import Signal
 from .multibandsignal import MultiBandSignal
 from .filter_class import Filter
 from ._filter import _filterbank_on_signal
-from dsptoolbox.generators import dirac
-from dsptoolbox.plots import general_plot
-from dsptoolbox._general_helpers import (
-    _get_normalized_spectrum, _check_format_in_path)
-from dsptoolbox._standard import _group_delay_direct
+from ..generators import dirac
+from ..plots import general_plot
+from .._general_helpers import _get_normalized_spectrum, _check_format_in_path
+from .._standard import _group_delay_direct
 
 
-class FilterBank():
+class FilterBank:
     """Standard template for filter banks containing filters, filters' initial
     values, metadata and some useful plotting methods.
 
     """
+
     # ======== Constructor and initializers ===================================
-    def __init__(self, filters: list = None, same_sampling_rate: bool = True,
-                 info: dict = None):
+    def __init__(
+        self,
+        filters: list | None = None,
+        same_sampling_rate: bool = True,
+        info: dict | None = None,
+    ):
         """FilterBank object saves multiple filters and some metadata.
         It also allows for easy filtering with multiple filters.
         Since the digital filters that are supported are linear systems,
@@ -57,18 +61,16 @@ class FilterBank():
         self.info = self.info | info
 
     def _generate_metadata(self):
-        """Generates the info dictionary with metadata about the FilterBank.
-
-        """
+        """Generates the info dictionary with metadata about the FilterBank."""
         self.info = {}
-        self.info['number_of_filters'] = self.number_of_filters
-        self.info['same_sampling_rate'] = self.same_sampling_rate
+        self.info["number_of_filters"] = self.number_of_filters
+        self.info["same_sampling_rate"] = self.same_sampling_rate
         if self.same_sampling_rate:
-            if hasattr(self, 'sampling_rate_hz'):
-                self.info['sampling_rate_hz'] = self.sampling_rate_hz
-        self.info['types_of_filters'] = \
-            tuple(set([f.info['filter_type']
-                       for f in self.filters]))
+            if hasattr(self, "sampling_rate_hz"):
+                self.info["sampling_rate_hz"] = self.sampling_rate_hz
+        self.info["types_of_filters"] = tuple(
+            set([f.info["filter_type"] for f in self.filters])
+        )
 
     def initialize_zi(self, number_of_channels: int = 1):
         """Initiates the zi of the filters for the given number of channels.
@@ -91,8 +93,9 @@ class FilterBank():
     def sampling_rate_hz(self, new_sampling_rate_hz):
         new_sampling_rate_hz = np.asarray(new_sampling_rate_hz).squeeze()
         if self.same_sampling_rate:
-            assert new_sampling_rate_hz.ndim == 0, \
-                'Sampling rate should be only an integer'
+            assert (
+                new_sampling_rate_hz.ndim == 0
+            ), "Sampling rate should be only an integer"
             self.__sampling_rate_hz = int(new_sampling_rate_hz)
         else:
             new_sampling_rate_hz = np.atleast_1d(new_sampling_rate_hz)
@@ -106,10 +109,9 @@ class FilterBank():
     def filters(self, new_filters):
         if new_filters is None:
             new_filters = []
-        if type(new_filters) == tuple:
+        if type(new_filters) is tuple:
             new_filters = list(new_filters)
-        assert type(new_filters) == list, \
-            'Filters have to be passed as a list'
+        assert type(new_filters) is list, "Filters have to be passed as a list"
         if new_filters:
             if self.same_sampling_rate:
                 self.sampling_rate_hz = new_filters[0].sampling_rate_hz
@@ -119,11 +121,13 @@ class FilterBank():
                     sr.append(f.sampling_rate_hz)
                 self.sampling_rate_hz = sr
             for ind, f in enumerate(new_filters):
-                assert type(f) == Filter, \
-                    f'Object at index {ind} is not a supported Filter'
+                assert (
+                    type(f) is Filter
+                ), f"Object at index {ind} is not a supported Filter"
                 if self.same_sampling_rate:
-                    assert f.sampling_rate_hz == self.sampling_rate_hz, \
-                        'Sampling rates do not match'
+                    assert (
+                        f.sampling_rate_hz == self.sampling_rate_hz
+                    ), "Sampling rates do not match"
         self.__filters = new_filters
         self._generate_metadata()
 
@@ -146,8 +150,7 @@ class FilterBank():
 
     @same_sampling_rate.setter
     def same_sampling_rate(self, new_same):
-        assert type(new_same) == bool, \
-            'same_sampling_rate must be a boolean'
+        assert type(new_same) is bool, "same_sampling_rate must be a boolean"
         self.__same_sampling_rate = new_same
 
     # ======== Add and remove =================================================
@@ -168,8 +171,9 @@ class FilterBank():
         else:
             fs = self.filters.copy()
             if self.same_sampling_rate:
-                assert self.sampling_rate_hz == filt.sampling_rate_hz, \
-                    'Sampling rates do not match'
+                assert (
+                    self.sampling_rate_hz == filt.sampling_rate_hz
+                ), "Sampling rates do not match"
             if index == -1:
                 fs.append(filt)
             else:
@@ -177,8 +181,9 @@ class FilterBank():
             self.filters = fs
         self._generate_metadata()
 
-    def remove_filter(self, index: int = -1, return_filter: bool = False) ->\
-            None | Filter:
+    def remove_filter(
+        self, index: int = -1, return_filter: bool = False
+    ) -> None | Filter:
         """Removes a filter from the filter bank.
 
         Parameters
@@ -191,11 +196,12 @@ class FilterBank():
             When `True`, the erased filter is returned. Default: `False`.
 
         """
-        assert self.filters, 'There are no filters to remove'
+        assert self.filters, "There are no filters to remove"
         if index == -1:
             index = len(self.filters) - 1
-        assert index in range(len(self.filters)), \
-            f'There is no filter at index {index}.'
+        assert index in range(
+            len(self.filters)
+        ), f"There is no filter at index {index}."
         n_f = self.filters.copy()
         f = n_f.pop(index)
         self.filters = n_f
@@ -212,24 +218,33 @@ class FilterBank():
 
         """
         new_order = np.array(new_order).squeeze()
-        assert new_order.ndim == 1, \
-            'Too many or too few dimensions are given in the new ' +\
-            'arrangement vector'
-        assert self.number_of_filters == len(new_order), \
-            'The number of filters does not match'
-        assert all(new_order < self.number_of_filters) \
-            and all(new_order >= 0), \
-            'Indexes of new filters have to be in ' +\
-            f'[0, {self.number_of_filters-1}]'
-        assert len(np.unique(new_order)) == len(new_order), \
-            'There are repeated indexes in the new order vector'
+        assert new_order.ndim == 1, (
+            "Too many or too few dimensions are given in the new "
+            + "arrangement vector"
+        )
+        assert self.number_of_filters == len(
+            new_order
+        ), "The number of filters does not match"
+        assert all(new_order < self.number_of_filters) and all(
+            new_order >= 0
+        ), (
+            "Indexes of new filters have to be in "
+            + f"[0, {self.number_of_filters - 1}]"
+        )
+        assert len(np.unique(new_order)) == len(
+            new_order
+        ), "There are repeated indexes in the new order vector"
         n_f = [self.filters[i] for i in new_order]
         self.filters = n_f
 
     # ======== Filtering ======================================================
-    def filter_signal(self, signal: Signal, mode: str = 'parallel',
-                      activate_zi: bool = False, zero_phase: bool = False) -> \
-            Signal | MultiBandSignal:
+    def filter_signal(
+        self,
+        signal: Signal,
+        mode: str = "parallel",
+        activate_zi: bool = False,
+        zero_phase: bool = False,
+    ) -> Signal | MultiBandSignal:
         """Applies the filter bank to a signal and returns a multiband signal
         or a `Signal` object.
         `'parallel'`: returns a `MultiBandSignal` object where each band is
@@ -260,39 +275,52 @@ class FilterBank():
             New signal after filtering.
 
         """
-        if type(signal) == MultiBandSignal:
+        if type(signal) is MultiBandSignal:
             raise TypeError(
-                'This method only supports Signal objects. Use ' +
-                'filter_multiband_signal() for multirate parallel filtering')
+                "This method only supports Signal objects. Use "
+                + "filter_multiband_signal() for multirate parallel filtering"
+            )
         mode = mode.lower()
-        assert mode in ('parallel', 'sequential', 'summed'), \
-            f'{mode} is not a valid mode. Use parallel, sequential or summed'
-        if mode in ('sequential', 'summed'):
-            assert self.same_sampling_rate, \
-                'Multirate filtering is not valid for sequential or summed ' +\
-                'filtering'
-        assert np.all(signal.sampling_rate_hz == self.sampling_rate_hz), \
-            'Sampling rates do not match'
+        assert mode in (
+            "parallel",
+            "sequential",
+            "summed",
+        ), f"{mode} is not a valid mode. Use parallel, sequential or summed"
+        if mode in ("sequential", "summed"):
+            assert self.same_sampling_rate, (
+                "Multirate filtering is not valid for sequential or summed "
+                + "filtering"
+            )
+        assert np.all(
+            signal.sampling_rate_hz == self.sampling_rate_hz
+        ), "Sampling rates do not match"
         if zero_phase:
-            assert not activate_zi, \
-                'Zero-phase filtering and zi cannot be used at ' +\
-                'the same time'
+            assert not activate_zi, (
+                "Zero-phase filtering and zi cannot be used at "
+                + "the same time"
+            )
         if activate_zi:
+            if not hasattr(self.filters[0], "zi"):
+                self.initialize_zi(signal.number_of_channels)
             if len(self.filters[0].zi) != signal.number_of_channels:
                 self.initialize_zi(signal.number_of_channels)
 
         new_sig = _filterbank_on_signal(
-            signal, self.filters,
+            signal,
+            self.filters,
             mode=mode,
             activate_zi=activate_zi,
             zero_phase=zero_phase,
-            same_sampling_rate=self.same_sampling_rate)
+            same_sampling_rate=self.same_sampling_rate,
+        )
         return new_sig
 
-    def filter_multiband_signal(self, mbsignal: MultiBandSignal,
-                                activate_zi: bool = False,
-                                zero_phase: bool = False) -> \
-            MultiBandSignal:
+    def filter_multiband_signal(
+        self,
+        mbsignal: MultiBandSignal,
+        activate_zi: bool = False,
+        zero_phase: bool = False,
+    ) -> MultiBandSignal:
         """Applies the filter bank to a `MultiBandSignal` and returns the
         output as a `MultiBandSignal` as well. Only `'parallel'` mode is
         supported.
@@ -316,13 +344,17 @@ class FilterBank():
             New signal after filtering.
 
         """
-        assert np.all(mbsignal.sampling_rate_hz == self.sampling_rate_hz), \
-            'Sampling rates do not match'
+        assert np.all(
+            mbsignal.sampling_rate_hz == self.sampling_rate_hz
+        ), "Sampling rates do not match"
         if zero_phase:
-            assert not activate_zi, \
-                'Zero-phase filtering and zi cannot be used at ' +\
-                'the same time'
+            assert not activate_zi, (
+                "Zero-phase filtering and zi cannot be used at "
+                + "the same time"
+            )
         if activate_zi:
+            if not hasattr(self.filters[0], "zi"):
+                self.initialize_zi(mbsignal.number_of_channels)
             if len(self.filters[0].zi) != mbsignal.number_of_channels:
                 self.initialize_zi(mbsignal.number_of_channels)
 
@@ -330,14 +362,21 @@ class FilterBank():
 
         for n in range(mbsignal.number_of_bands):
             new_sig.bands[n] = self.filters[n].filter_signal(
-                mbsignal.bands[n], channels=None, activate_zi=activate_zi,
-                zero_phase=zero_phase)
+                mbsignal.bands[n],
+                channels=None,
+                activate_zi=activate_zi,
+                zero_phase=zero_phase,
+            )
         return new_sig
 
     # ======== Get impulse ====================================================
-    def get_ir(self, mode: str = 'parallel', length_samples: int = 2048,
-               test_zi: bool = False, zero_phase: bool = False) -> \
-            Signal | MultiBandSignal:
+    def get_ir(
+        self,
+        mode: str = "parallel",
+        length_samples: int = 2048,
+        test_zi: bool = False,
+        zero_phase: bool = False,
+    ) -> Signal | MultiBandSignal:
         """Returns impulse response from the filter bank.
 
         Parameters
@@ -363,17 +402,21 @@ class FilterBank():
         """
         # No plotting for multirate system
         if not self.same_sampling_rate:
-            warn('Plotting for multirate FilterBank is not supported, ' +
-                 'skipping IR generation')
+            warn(
+                "Plotting for multirate FilterBank is not supported, "
+                + "skipping IR generation"
+            )
             return None
         # Obtain biggest filter order from FilterBank
         max_order = 0
         for b in self.filters:
-            max_order = max(max_order, b.info['order'])
+            max_order = max(max_order, b.info["order"])
         if max_order > length_samples:
-            warn(f'Filter order {max_order} is longer than {length_samples}.' +
-                 'The length will be adapted to be 100 samples longer than' +
-                 ' the longest filter')
+            warn(
+                f"Filter order {max_order} is longer than {length_samples}."
+                + "The length will be adapted to be 100 samples longer than"
+                + " the longest filter"
+            )
             length_samples = max_order + 100
 
         # Sampling rate
@@ -382,43 +425,46 @@ class FilterBank():
         # Impulse
         d = dirac(
             length_samples=length_samples,
-            number_of_channels=1, sampling_rate_hz=fs_hz)
+            number_of_channels=1,
+            sampling_rate_hz=fs_hz,
+        )
 
         # Filtering
         ir = self.filter_signal(
-            d, mode, activate_zi=test_zi, zero_phase=zero_phase)
+            d, mode, activate_zi=test_zi, zero_phase=zero_phase
+        )
         return ir
 
     # ======== Prints and plots ===============================================
     def show_info(self):
-        """Show information about the filter bank.
-
-        """
+        """Show information about the filter bank."""
         print(self._get_metadata_str())
 
     def _get_metadata_str(self):
-        txt = ''
+        txt = ""
         for k in self.info:
-            txt += \
-                f""" | {str(k).replace('_', ' ').
-                        capitalize()}: {self.info[k]}"""
-        txt = 'Filter Bank:' + txt
-        txt += '\n'
-        txt += '–'*len(txt)
+            txt += f""" | {str(k).replace('_', ' ').
+                           capitalize()}: {self.info[k]}"""
+        txt = "Filter Bank:" + txt
+        txt += "\n"
+        txt += "–" * len(txt)
         for ind, f1 in enumerate(self.filters):
-            txt += '\n'
-            txt += f'Filter {ind}:'
+            txt += "\n"
+            txt += f"Filter {ind}:"
             for kf in f1.info:
-                if kf == 'ba':
+                if kf == "ba":
                     continue
-                txt += \
-                    f""" | {str(kf).replace('_', ' ').
-                            capitalize()}: {f1.info[kf]}"""
+                txt += f""" | {str(kf).replace('_', ' ').
+                               capitalize()}: {f1.info[kf]}"""
         return txt
 
-    def plot_magnitude(self, mode: str = 'parallel', range_hz=[20, 20e3],
-                       length_samples: int = 2048, test_zi: bool = False) ->\
-            tuple[Figure, Axes]:
+    def plot_magnitude(
+        self,
+        mode: str = "parallel",
+        range_hz=[20, 20e3],
+        length_samples: int = 2048,
+        test_zi: bool = False,
+    ) -> tuple[Figure, Axes]:
         """Plots the magnitude response of each filter.
 
         Parameters
@@ -447,71 +493,99 @@ class FilterBank():
         """
         # No plotting for multirate system
         if not self.same_sampling_rate:
-            warn('Plotting for multirate FilterBank is not supported, ' +
-                 'skipping plots')
+            warn(
+                "Plotting for multirate FilterBank is not supported, "
+                + "skipping plots"
+            )
             return None
         # Length handling
         max_order = 0
         for b in self.filters:
-            max_order = max(max_order, b.info['order'])
+            max_order = max(max_order, b.info["order"])
         if max_order > length_samples:
-            warn(f'Filter order {max_order} is longer than {length_samples}.' +
-                 ' The length will be adapted to be 100 samples longer than' +
-                 ' the longest filter')
+            warn(
+                f"Filter order {max_order} is longer than {length_samples}."
+                + " The length will be adapted to be 100 samples longer than"
+                + " the longest filter"
+            )
             length_samples = max_order + 100
 
         # Impulse
         d = dirac(
             length_samples=length_samples,
-            number_of_channels=1, sampling_rate_hz=self.sampling_rate_hz)
+            number_of_channels=1,
+            sampling_rate_hz=self.sampling_rate_hz,
+        )
 
         # Filtering and plot
-        if mode == 'parallel':
-            bs = self.filter_signal(d, mode='parallel', activate_zi=test_zi)
+        if mode == "parallel":
+            bs = self.filter_signal(d, mode="parallel", activate_zi=test_zi)
             specs = []
             for b in bs.bands:
-                b.set_spectrum_parameters(method='standard')
+                b.set_spectrum_parameters(method="standard")
                 f, sp = b.get_spectrum()
                 f, sp = _get_normalized_spectrum(
-                    f, sp,
-                    f_range_hz=range_hz,
-                    normalize=None)
+                    f, sp, f_range_hz=range_hz, normalize=None
+                )
                 specs.append(np.squeeze(sp))
             specs = np.array(specs).T
-            if np.min(specs) < np.max(specs)-50:
-                range_y = [np.max(specs)-50, np.max(specs)+2]
+            if np.min(specs) < np.max(specs) - 50:
+                range_y = [np.max(specs) - 50, np.max(specs) + 2]
             else:
                 range_y = None
-            fig, ax = general_plot(f, specs, range_hz, ylabel='Magnitude / dB',
-                                   returns=True,
-                                   labels=[f'Filter {h}'
-                                           for h in range(bs.number_of_bands)],
-                                   range_y=range_y,
-                                   tight_layout=False)
-        elif mode == 'sequential':
-            bs = self.filter_signal(d, mode='sequential', activate_zi=test_zi)
-            bs.set_spectrum_parameters(method='standard')
+            fig, ax = general_plot(
+                f,
+                specs,
+                range_hz,
+                ylabel="Magnitude / dB",
+                returns=True,
+                labels=[f"Filter {h}" for h in range(bs.number_of_bands)],
+                range_y=range_y,
+                tight_layout=False,
+            )
+        elif mode == "sequential":
+            bs = self.filter_signal(d, mode="sequential", activate_zi=test_zi)
+            bs.set_spectrum_parameters(method="standard")
             f, sp = bs.get_spectrum()
             f, sp = _get_normalized_spectrum(
-                f, np.squeeze(sp), f_range_hz=range_hz, normalize=None)
+                f, np.squeeze(sp), f_range_hz=range_hz, normalize=None
+            )
             fig, ax = general_plot(
-                f, sp, range_hz, ylabel='Magnitude / dB', returns=True,
-                labels=[f'Sequential - Channel {n}'
-                        for n in range(bs.number_of_channels)])
-        elif mode == 'summed':
-            bs = self.filter_signal(d, mode='summed', activate_zi=test_zi)
-            bs.set_spectrum_parameters(method='standard')
+                f,
+                sp,
+                range_hz,
+                ylabel="Magnitude / dB",
+                returns=True,
+                labels=[
+                    f"Sequential - Channel {n}"
+                    for n in range(bs.number_of_channels)
+                ],
+            )
+        elif mode == "summed":
+            bs = self.filter_signal(d, mode="summed", activate_zi=test_zi)
+            bs.set_spectrum_parameters(method="standard")
             f, sp = bs.get_spectrum()
             f, sp = _get_normalized_spectrum(
-                f, np.squeeze(sp), f_range_hz=range_hz, normalize=None)
+                f, np.squeeze(sp), f_range_hz=range_hz, normalize=None
+            )
             fig, ax = general_plot(
-                f, sp, range_hz, ylabel='Magnitude / dB', returns=True,
-                labels=['Summed'])
+                f,
+                sp,
+                range_hz,
+                ylabel="Magnitude / dB",
+                returns=True,
+                labels=["Summed"],
+            )
         return fig, ax
 
-    def plot_phase(self, mode: str = 'parallel', range_hz=[20, 20e3],
-                   unwrap: bool = False, length_samples: int = 2048,
-                   test_zi: bool = False) -> tuple[Figure, Axes]:
+    def plot_phase(
+        self,
+        mode: str = "parallel",
+        range_hz=[20, 20e3],
+        unwrap: bool = False,
+        length_samples: int = 2048,
+        test_zi: bool = False,
+    ) -> tuple[Figure, Axes]:
         """Plots the phase response of each filter.
 
         Parameters
@@ -542,27 +616,33 @@ class FilterBank():
         """
         # No plotting for multirate system
         if not self.same_sampling_rate:
-            warn('Plotting for multirate FilterBank is not supported, ' +
-                 'skipping plots')
+            warn(
+                "Plotting for multirate FilterBank is not supported, "
+                + "skipping plots"
+            )
             return None
         # Length handling
         max_order = 0
         for b in self.filters:
-            max_order = max(max_order, b.info['order'])
+            max_order = max(max_order, b.info["order"])
         if max_order > length_samples:
-            warn(f'Filter order {max_order} is longer than {length_samples}.' +
-                 ' The length will be adapted to be 100 samples longer than' +
-                 ' the longest filter')
+            warn(
+                f"Filter order {max_order} is longer than {length_samples}."
+                + " The length will be adapted to be 100 samples longer than"
+                + " the longest filter"
+            )
             length_samples = max_order + 100
 
         # Generate impulse
         d = dirac(
             length_samples=length_samples,
-            number_of_channels=1, sampling_rate_hz=self.sampling_rate_hz)
+            number_of_channels=1,
+            sampling_rate_hz=self.sampling_rate_hz,
+        )
 
         # Plot
-        if mode == 'parallel':
-            bs = self.filter_signal(d, mode='parallel', activate_zi=test_zi)
+        if mode == "parallel":
+            bs = self.filter_signal(d, mode="parallel", activate_zi=test_zi)
             phase = []
             f = bs.bands[0].get_spectrum()[0]
             for b in bs.bands:
@@ -570,37 +650,55 @@ class FilterBank():
             phase = np.squeeze(np.array(phase).T)
             if unwrap:
                 phase = np.unwrap(phase, axis=0)
-            fig, ax = general_plot(f, phase, range_hz, ylabel='Phase / rad',
-                                   returns=True,
-                                   labels=[f'Filter {h}'
-                                           for h in range(bs.number_of_bands)],
-                                   tight_layout=False)
-        elif mode == 'sequential':
-            bs = self.filter_signal(d, mode='sequential', activate_zi=test_zi)
+            fig, ax = general_plot(
+                f,
+                phase,
+                range_hz,
+                ylabel="Phase / rad",
+                returns=True,
+                labels=[f"Filter {h}" for h in range(bs.number_of_bands)],
+                tight_layout=False,
+            )
+        elif mode == "sequential":
+            bs = self.filter_signal(d, mode="sequential", activate_zi=test_zi)
             f, sp = bs.get_spectrum()
             ph = np.angle(sp)
             if unwrap:
                 ph = np.unwrap(ph, axis=0)
             fig, ax = general_plot(
-                f, ph, range_hz, ylabel='Phase / rad',
+                f,
+                ph,
+                range_hz,
+                ylabel="Phase / rad",
                 returns=True,
-                labels=[f'Sequential - Channel {n}'
-                        for n in range(bs.number_of_channels)])
-        elif mode == 'summed':
-            bs = self.filter_signal(d, mode='summed', activate_zi=test_zi)
+                labels=[
+                    f"Sequential - Channel {n}"
+                    for n in range(bs.number_of_channels)
+                ],
+            )
+        elif mode == "summed":
+            bs = self.filter_signal(d, mode="summed", activate_zi=test_zi)
             f, sp = bs.get_spectrum()
             ph = np.angle(sp)
             if unwrap:
                 ph = np.unwrap(ph, axis=0)
             fig, ax = general_plot(
-                f, ph, range_hz, ylabel='Phase / rad',
+                f,
+                ph,
+                range_hz,
+                ylabel="Phase / rad",
                 returns=True,
-                labels=['Summed'])
+                labels=["Summed"],
+            )
         return fig, ax
 
-    def plot_group_delay(self, mode: str = 'parallel', range_hz=[20, 20e3],
-                         length_samples: int = 2048, test_zi: bool = False) ->\
-            tuple[Figure, Axes]:
+    def plot_group_delay(
+        self,
+        mode: str = "parallel",
+        range_hz=[20, 20e3],
+        length_samples: int = 2048,
+        test_zi: bool = False,
+    ) -> tuple[Figure, Axes]:
         """Plots the phase response of each filter.
 
         Parameters
@@ -629,59 +727,82 @@ class FilterBank():
         """
         # No plotting for multirate system
         if not self.same_sampling_rate:
-            warn('Plotting for multirate FilterBank is not supported, ' +
-                 'skipping plots')
+            warn(
+                "Plotting for multirate FilterBank is not supported, "
+                + "skipping plots"
+            )
             return None
         # Length handling
         max_order = 0
         for b in self.filters:
-            max_order = max(max_order, b.info['order'])
+            max_order = max(max_order, b.info["order"])
         if max_order > length_samples:
-            warn(f'Filter order {max_order} is longer than {length_samples}.' +
-                 ' The length will be adapted to be 100 samples longer than' +
-                 ' the longest filter')
+            warn(
+                f"Filter order {max_order} is longer than {length_samples}."
+                + " The length will be adapted to be 100 samples longer than"
+                + " the longest filter"
+            )
             length_samples = max_order + 100
 
         # Impulse
         d = dirac(
             length_samples=length_samples,
-            number_of_channels=1, sampling_rate_hz=self.sampling_rate_hz)
+            number_of_channels=1,
+            sampling_rate_hz=self.sampling_rate_hz,
+        )
 
         # Plot
-        if mode == 'parallel':
-            bs = self.filter_signal(d, mode='parallel', activate_zi=test_zi)
+        if mode == "parallel":
+            bs = self.filter_signal(d, mode="parallel", activate_zi=test_zi)
             gd = []
             f = bs.bands[0].get_spectrum()[0]
             for b in bs.bands:
-                gd.append(_group_delay_direct(
-                    np.squeeze(b.get_spectrum()[1]), delta_f=f[1]-f[0]))
-            gd = np.squeeze(np.array(gd).T)*1e3
-            fig, ax = general_plot(f, gd, range_hz, ylabel='Group delay / ms',
-                                   returns=True,
-                                   labels=[f'Filter {h}'
-                                           for h in range(bs.number_of_bands)],
-                                   tight_layout=False)
-        elif mode == 'sequential':
-            bs = self.filter_signal(d, mode='sequential', activate_zi=test_zi)
-            f, sp = bs.get_spectrum()
-            gd = _group_delay_direct(sp.squeeze(), f[1]-f[0])*1e3
+                gd.append(
+                    _group_delay_direct(
+                        np.squeeze(b.get_spectrum()[1]), delta_f=f[1] - f[0]
+                    )
+                )
+            gd = np.squeeze(np.array(gd).T) * 1e3
             fig, ax = general_plot(
-                f, gd[..., None], range_hz, ylabel='Group delay / ms',
+                f,
+                gd,
+                range_hz,
+                ylabel="Group delay / ms",
                 returns=True,
-                labels=[f'Sequential - Channel {n}'
-                        for n in range(bs.number_of_channels)])
-        elif mode == 'summed':
-            bs = self.filter_signal(d, mode='summed', activate_zi=test_zi)
+                labels=[f"Filter {h}" for h in range(bs.number_of_bands)],
+                tight_layout=False,
+            )
+        elif mode == "sequential":
+            bs = self.filter_signal(d, mode="sequential", activate_zi=test_zi)
             f, sp = bs.get_spectrum()
-            gd = _group_delay_direct(sp.squeeze(), f[1]-f[0])*1e3
+            gd = _group_delay_direct(sp.squeeze(), f[1] - f[0]) * 1e3
             fig, ax = general_plot(
-                f, gd[..., None], range_hz, ylabel='Group delay / ms',
+                f,
+                gd[..., None],
+                range_hz,
+                ylabel="Group delay / ms",
                 returns=True,
-                labels=['Summed'])
+                labels=[
+                    f"Sequential - Channel {n}"
+                    for n in range(bs.number_of_channels)
+                ],
+            )
+        elif mode == "summed":
+            bs = self.filter_signal(d, mode="summed", activate_zi=test_zi)
+            f, sp = bs.get_spectrum()
+            gd = _group_delay_direct(sp.squeeze(), f[1] - f[0]) * 1e3
+            fig, ax = general_plot(
+                f,
+                gd[..., None],
+                range_hz,
+                ylabel="Group delay / ms",
+                returns=True,
+                labels=["Summed"],
+            )
         return fig, ax
 
     # ======== Saving and export ==============================================
-    def save_filterbank(self, path: str = 'filterbank'):
+    def save_filterbank(self, path: str = "filterbank"):
         """Saves the FilterBank object as a pickle.
 
         Parameters
@@ -692,8 +813,8 @@ class FilterBank():
             (local folder, object named filterbank).
 
         """
-        path = _check_format_in_path(path, 'pkl')
-        with open(path, 'wb') as data_file:
+        path = _check_format_in_path(path, "pkl")
+        with open(path, "wb") as data_file:
             dump(self, data_file, HIGHEST_PROTOCOL)
 
     def copy(self):
