@@ -20,12 +20,12 @@ Mixed:
 - `fw_snr_seg()`
 
 """
-from .distances import (log_spectral, itakura_saito, snr, si_sdr, fw_snr_seg)
+from .distances import log_spectral, itakura_saito, snr, si_sdr, fw_snr_seg
 
 __all__ = [
-    'log_spectral',
-    'itakura_saito',
-    'snr',
-    'si_sdr',
-    'fw_snr_seg',
+    "log_spectral",
+    "itakura_saito",
+    "snr",
+    "si_sdr",
+    "fw_snr_seg",
 ]
