@@ -3,12 +3,13 @@ Backend for distance measures
 """
 import numpy as np
 from scipy.integrate import simpson
-from dsptoolbox._general_helpers import _compute_number_frames, _pad_trim
-from dsptoolbox._standard import _rms
+from .._general_helpers import _compute_number_frames, _pad_trim
+from .._standard import _rms
 
 
-def _log_spectral_distance(x: np.ndarray, y: np.ndarray, f: np.ndarray) \
-        -> float:
+def _log_spectral_distance(
+    x: np.ndarray, y: np.ndarray, f: np.ndarray
+) -> float:
     """Computes log spectral distance between two signals.
 
     Parameters
@@ -26,15 +27,15 @@ def _log_spectral_distance(x: np.ndarray, y: np.ndarray, f: np.ndarray) \
         Log spectral distance.
 
     """
-    assert x.shape == y.shape, \
-        'Power spectra have different lengths'
-    integral = simpson((10*np.log10(x/y))**2, f)
+    assert x.shape == y.shape, "Power spectra have different lengths"
+    integral = simpson((10 * np.log10(x / y)) ** 2, f)
     log_spec = np.sqrt(integral)
     return log_spec
 
 
-def _itakura_saito_measure(x: np.ndarray, y: np.ndarray, f: np.ndarray) \
-        -> float:
+def _itakura_saito_measure(
+    x: np.ndarray, y: np.ndarray, f: np.ndarray
+) -> float:
     """Computes log spectral distance between two signals.
 
     Parameters
@@ -52,9 +53,8 @@ def _itakura_saito_measure(x: np.ndarray, y: np.ndarray, f: np.ndarray) \
         Itakura Saito measure.
 
     """
-    assert x.shape == y.shape, \
-        'Power spectra have different lengths'
-    ism = simpson(x/y - np.log10(x/y) - 1, f)
+    assert x.shape == y.shape, "Power spectra have different lengths"
+    ism = simpson(x / y - np.log10(x / y) - 1, f)
     return ism
 
 
@@ -74,7 +74,7 @@ def _snr(s: np.ndarray, n: np.ndarray) -> float:
         SNR between signals.
 
     """
-    return 20*np.log10(_rms(s)/_rms(n))
+    return 20 * np.log10(_rms(s) / _rms(n))
 
 
 def _sisdr(s: np.ndarray, shat: np.ndarray) -> float:
@@ -93,15 +93,21 @@ def _sisdr(s: np.ndarray, shat: np.ndarray) -> float:
         SI-SDR value between two signals.
 
     """
-    alpha = (s @ shat)/(s @ s)
-    sisdr = 10*np.log10(np.sum((alpha*s)**2) / np.sum((alpha*s - shat)**2))
+    alpha = (s @ shat) / (s @ s)
+    sisdr = 10 * np.log10(
+        np.sum((alpha * s) ** 2) / np.sum((alpha * s - shat) ** 2)
+    )
     return sisdr
 
 
-def _fw_snr_seg_per_channel(x: np.ndarray, xhat: np.ndarray,
-                            snr_range_db: np.ndarray, gamma: float,
-                            time_window: np.ndarray,
-                            step_samples: int) -> float:
+def _fw_snr_seg_per_channel(
+    x: np.ndarray,
+    xhat: np.ndarray,
+    snr_range_db: np.ndarray,
+    gamma: float,
+    time_window: np.ndarray,
+    step_samples: int,
+) -> float:
     """This function gets an original signal x and a modified signal xhat
     and computes the frequency-weighted segmental SNR according to
     Y. Hu and P. C. Loizou,
@@ -140,20 +146,21 @@ def _fw_snr_seg_per_channel(x: np.ndarray, xhat: np.ndarray,
     eps = 1e-30  # Some small number for the logarithm function
     length_signal = len(x)
     length_window = len(time_window)
-    n_frames, pad_samples = \
-        _compute_number_frames(length_window, step_samples, length_signal)
-    x = _pad_trim(x, length_signal+pad_samples)
-    xhat = _pad_trim(xhat, length_signal+pad_samples)
+    n_frames, pad_samples = _compute_number_frames(
+        length_window, step_samples, length_signal
+    )
+    x = _pad_trim(x, length_signal + pad_samples)
+    xhat = _pad_trim(xhat, length_signal + pad_samples)
 
     fw_snr_seg = 0
     position = 0
     # Loop for time frames
     for _ in range(n_frames):
         # Time signals
-        x_m = x[position:position+length_window, :]
-        xhat_m = xhat[position:position+length_window, :]
+        x_m = x[position : position + length_window, :]
+        xhat_m = xhat[position : position + length_window, :]
         # "Collectors"
-        weights_jm = np.zeros(length_window//2+1)
+        weights_jm = np.zeros(length_window // 2 + 1)
         snr_jm = np.zeros_like(weights_jm)
         # Loop for bands
         for ib in range(x.shape[1]):
@@ -167,7 +174,7 @@ def _fw_snr_seg_per_channel(x: np.ndarray, xhat: np.ndarray,
             X_jm /= np.sum(X_jm)
             Xhat_jm /= np.sum(Xhat_jm)
 
-            snr_jm += (np.log10(X_jm**2 / (X_jm - Xhat_jm + eps)**2)*W_jm)
+            snr_jm += np.log10(X_jm**2 / (X_jm - Xhat_jm + eps) ** 2) * W_jm
             weights_jm += W_jm
 
         # Mean SNR over all frequencies
