@@ -9,11 +9,12 @@ This module contains some utility signal generators. Choose from:
 - `harmonic()`
 
 """
-from .generators import chirp, noise, dirac, harmonic
+from .generators import chirp, noise, dirac, harmonic, oscillator
 
 __all__ = [
-    'chirp',
-    'noise',
-    'dirac',
-    'harmonic',
+    "chirp",
+    "noise",
+    "dirac",
+    "harmonic",
+    "oscillator",
 ]
