@@ -10,12 +10,16 @@ styling.
 - `show()`
 
 """
-from .plots import (general_matrix_plot, general_plot,
-                    general_subplots_line, show)
+from .plots import (
+    general_matrix_plot,
+    general_plot,
+    general_subplots_line,
+    show,
+)
 
 __all__ = [
-    'general_matrix_plot',
-    'general_plot',
-    'general_subplots_line',
-    'show',
+    "general_matrix_plot",
+    "general_plot",
+    "general_subplots_line",
+    "show",
 ]
